/*

  This file contains three things:
  * MCE.conv_table (variable)
  * MCE.converters (object, contains in2m(), etc)
  * MCE.adjusters (object, contains unit adjuster functions)

*/

/*

TO DO:
 - metric weights => imperial weights
 - surfaces
 - volumes
 - update example page with speeds plus everything above
 - example page: adaptive results -- in2m, m2in, kmph2miph, miph2kmph, lb2kg
*/

if (MCE!=undefined && MCE.applicationName=='Converter') {

/*
  This string defines which unit is parsed by what converter.
  
  Notes:
  1. Since we separate units when parsing the selection by spaces,
     it is safe to also separate them by spaces here. Therefore the format
     of this string is ((unit)(<space>)*(converter)(<space>)*)*
  2. The converters are functions, defined in MCE.converters.
  3. Make sure there is a backslash at the end of every line in this string.
     A backslash tells the JavaScript interpreter the line is *continued*
     on the next line (i.e. the lines are simply concatenated in the final
     string, no line breaks result in the end). Please note this behaviour is
     variable in JavaScript depending on the browser; luckily we don't have to
     worry about that in this environment. :)
  4. The selection parser is case-insensitive, so all units in here *MUST* be
     in small letters. For instance "C" as a unit in the string below
     would NOT match "15 C" in the selection, but "c" does.
*/

MCE.conv_table=new String("\
  \"               in2m \
  in               in2m \
  inch             in2m \
  inches           in2m \
  -inch            in2m \
  -inches          in2m \
  \
  mil              mil2m \
  \
  mi               mi2m \
  mile             mi2m \
  miles            mi2m \
  -mile            mi2m \
  -miles           mi2m \
  \
  ft               ft2m \
  foot             ft2m \
  feet             ft2m \
  -foot            ft2m \
  -feet            ft2m \
  \'               ft2m \
  \
  yard             yd2m \
  yards            yd2m \
  -yard            yd2m \
  -yards           yd2m \
  yd               yd2m \
  \
  ounce            oz2kg \
  oz               oz2kg \
  ounces           oz2kg \
  -ounce           oz2kg \
  -ounces          oz2kg \
  \
  pound            lb2kg \
  pounds           lb2kg \
  -pound           lb2kg \
  -pounds          lb2kg \
  lb               lb2kg \
  lbs              lb2kg \
  \
  stone            stone2all \
  stones           stone2all \
  -stone           stone2all \
  -stones          stone2all \
  \
  kg               kg2lb \
  kgs              kg2lb \
  kilogram         kg2lb \
  kilograms        kg2lb \
  \
  g                g2oz \
  gram             g2oz \
  grams            g2oz \
  \
  m                m2in \
  meter            m2in \
  meters           m2in \
  metre            m2in \
  metres           m2in \
  \
  mm               mm2in \
  millimetre       mm2in \
  millimetres      mm2in \
  millimeter       mm2in \
  millimeters      mm2in \
  \
  km               km2in \
  kilometre        km2in \
  kilometres       km2in \
  kilometer        km2in \
  kilometers       km2in \
  \
  "+String.fromCharCode(181)+"m               microm2in \
  micrometre       microm2in \
  micrometres      microm2in \
  micrometer       microm2in \
  micrometers      microm2in \
  micron           microm2in \
  microns          microm2in \
  \
  cm               cm2in \
  centimetre       cm2in \
  centimetres      cm2in \
  centimeter       cm2in \
  centimeters      cm2in \
  \
  "+String.fromCharCode(176)+"celsius         c2f \
  "+String.fromCharCode(176)+"celcius         c2f \
  "+String.fromCharCode(176)+"c               c2f \
  "+String.fromCharCode(176)+"_celsius        c2f \
  "+String.fromCharCode(176)+"_celcius        c2f \
  "+String.fromCharCode(176)+"_c              c2f \
  \u00BAcelsius         c2f \
  \u00BAcelcius         c2f \
  \u00BAc               c2f \
  \u00BA_celsius        c2f \
  \u00BA_celcius        c2f \
  \u00BA_c              c2f \
  "+ // DO NOT change the order here! Moving 'degree_c' up will kill the other matches!
  "\
  c                c2f \
  celsius          c2f \
  celcius          c2f \
  centigrade       c2f \
  centigrades      c2f \
  degree_celsius   c2f \
  degrees_celsius  c2f \
  degree_centigrade  c2f \
  degrees_centigrade c2f \
  degree_c         c2f \
  degrees_c        c2f \
  \
  "+String.fromCharCode(176)+"f               f2c \
  "+String.fromCharCode(176)+"fahrenheit      f2c \
  "+String.fromCharCode(176)+"_fahrenheit     f2c \
  "+String.fromCharCode(176)+"_f              f2c \
  \u00BAf               f2c \
  \u00BAfahrenheit      f2c \
  \u00BA_fahrenheit     f2c \
  \u00BA_f              f2c \
  f                  f2c \
  fahrenheit         f2c \
  degree_fahrenheit  f2c \
  degrees_fahrenheit f2c \
  degree_f           f2c \
  degrees_f          f2c \
  \
  mph              miph2kmph \
  mi/h             miph2kmph \
  mile/h           miph2kmph \
  miles/h          miph2kmph \
  mile/hour        miph2kmph \
  miles/hour       miph2kmph \
  miles_per_hour   miph2kmph \
  mile_per_hour    miph2kmph \
  mile_an_hour     miph2kmph \
  miles_an_hour    miph2kmph \
  \
  km/h             kmph2miph \
  kmph             kmph2miph \
  kilometers/hour  kmph2miph \
  kilometres/hour  kmph2miph \
  kilometers_per_hour kmph2miph \
  kilometres_per_hour kmph2miph \
  kilometer_per_hour kmph2miph \
  kilometre_per_hour kmph2miph \
  \
  degree           deg2all \
  degrees          deg2all \
  deg              deg2all \
  "+String.fromCharCode(176)+"            deg2all \
  \
  m/s              mps2all \
  mps              mps2all \
  meters_per_second mps2all \
  metres_per_second mps2all \
  meter_per_second mps2all \
  metre_per_second mps2all \
  \
  km/s              kmps2all \
  kmps              kmps2all \
  kilometers_per_second kmps2all \
  kilometres_per_second kmps2all \
  kilometer_per_second kmps2all \
  kilometre_per_second kmps2all \
  \
  l                litre2gallon \
  liter            litre2gallon \
  liters           litre2gallon \
  litre            litre2gallon \
  litres           litre2gallon \
  \
  gallon           gallon2litre \
  gallons          gallon2litre \
  gal              gallon2litre \
  \
  fluid_ounce      floz2ml \
  fluid_ounces     floz2ml \
  fl._oz.          floz2ml \
  fl._oz           floz2ml \
  fl_oz.           floz2ml \
  fl.oz.           floz2ml \
  fl_oz            floz2ml \
  floz             floz2ml \
  fluidounce       floz2ml \
  fluidounces      floz2ml \
  \
  ml               ml2floz \
  milliliter       ml2floz \
  milliliters      ml2floz \
  mililiter        ml2floz \
  mililiters       ml2floz \
  \
  pa               pa2psi \
  pascal           pa2psi \
  pascals          pa2psi \
  kpa              kpa2psi \
  psi              psi2kpa \
  \
  internal_time    internal_time \
  \
  square_foot      sqft2sqm \
  square_feet      sqft2sqm \
  square_ft        sqft2sqm \
  sq_ft            sqft2sqm \
  sq._ft.          sqft2sqm \
  sq_ft.           sqft2sqm \
  sq._ft           sqft2sqm \
  sq_feet          sqft2sqm \
  sq_foot          sqft2sqm \
  sf               sqft2sqm \
  foot"+String.fromCharCode(178)+" sqft2sqm \
  feet"+String.fromCharCode(178)+" sqft2sqm \
  ft"+String.fromCharCode(178)+" sqft2sqm \
  ft^2             sqft2sqm \
  foot^2           sqft2sqm \
  feet^2           sqft2sqm \
  \
  square_meters    sqm2sqft \
  square_metres    sqm2sqft \
  square_meter     sqm2sqft \
  square_metre     sqm2sqft \
  m"+String.fromCharCode(178)+" sqm2sqft \
  meter"+String.fromCharCode(178)+" sqm2sqft \
  metre"+String.fromCharCode(178)+" sqm2sqft \
  meters"+String.fromCharCode(178)+" sqm2sqft \
  metres"+String.fromCharCode(178)+" sqm2sqft \
  m^2              sqm2sqft \
  meter^2          sqm2sqft \
  metre^2          sqm2sqft \
  meters^2         sqm2sqft \
  metres^2         sqm2sqft \
  \
  cc               cc2cuin \
  cm^3             cc2cuin \
  cm"+String.fromCharCode(179)+" cc2cuin \
  cu_in            cuin2cc \
  in^3             cuin2cc \
  in"+String.fromCharCode(179)+" cuin2cc \
  \
  mpg              mpg2lptm \
  miles_per_gallon mpg2lptm \
  \
  nm               nm2ftlb \
  newton_metre     nm2ftlb \
  newton_metres    nm2ftlb \
  newton_meter     nm2ftlb \
  newton_meters    nm2ftlb \
");
/*

  ====================================================================
                          THE CONVERTERS
  ====================================================================
  
  Ok, now that we know which unit is parsed with what converter, let's
  define the converters themselves. If you add extra converters, please
  make sure the order of the functions below is the same as in the string
  above.
  
*/

MCE.converters={
	in2m:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v*0.0254;
	  result.inUnit="in";
	  result.outUnit="m";
	  return result;
	},

	mil2m:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v*0.0000254;
	  result.inUnit="mil";
	  result.outUnit="m";
	  return result;
	},

	mi2m:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v*1609.344;
	  result.inUnit="miles";
	  result.outUnit="m";
	  return result;
	},

	ft2m:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v*0.3048;
	  result.inUnit="ft";
	  result.outUnit="m";
	  return result;
	},

	yd2m:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v*0.9144;
	  result.inUnit="yd";
	  result.outUnit="m";
	  return result;
	},

	oz2kg:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v*0.0283;
	  result.inUnit="oz";
	  result.outUnit="kg";
	  return result;
	},

	lb2kg:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v*0.45359237;
	  result.inUnit="lbs";
	  result.outUnit="kg";
	  return result;
	},

	stone2all:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.inUnit="stone";
	  if (MCE.pref_util.getPref('pref_metric')) {
	    result.outValue=v*6.35029318;
	    result.outUnit="kg";
	  } else {
	    result.outValue=v*14;
	    result.outUnit="lbs";
	  }
	  return result;
	},

	kg2lb:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v/0.45359237;
	  result.inUnit="kg";
	  result.outUnit="lbs";
	  return result;
	},

	g2oz:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v/28.3;
	  result.inUnit="g";
	  result.outUnit="oz";
	  return result;
	},

	m2in:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v*1000/25.4;
	  result.inUnit="m";
	  result.outUnit="in";
	  return result;
	},

	mm2in:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v/25.4;
	  result.inUnit="mm";
	  result.outUnit="in";
	  return result;
	},

	km2in:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=(v/0.0254)*1000;
	  result.inUnit="km";
	  result.outUnit="in";
	  return result;
	},

	microm2in:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v/25400;
	  result.inUnit=String.fromCharCode(181)+"m";
	  result.outUnit="in";
	  return result;
	},

	cm2in:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v/2.54;
	  result.inUnit="cm";
	  result.outUnit="in";
	  return result;
	},

	c2f:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=Math.round((v*1.8+32)*10)/10;
	  result.inUnit=String.fromCharCode(176)+"C";
	  result.outUnit=String.fromCharCode(176)+"F";
	  return result;
	},

	f2c:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=Math.round(10*(v-32)/1.8)/10;
	  result.inUnit=String.fromCharCode(176)+"F";
	  result.outUnit=String.fromCharCode(176)+"C";
	  return result;
	},

	miph2kmph:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  if (v<2) {
	    result.outValue=Math.round(v*16093.44)/10000;
	  } else if (v < 10) {
	    result.outValue=Math.round(v*160.9344)/100;
	  } else {
	    result.outValue=Math.round(v*1.609344);
	  }
	  result.inUnit="mph";
	  result.outUnit="km/h";
	  return result;
	},

	kmph2miph:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  if (v<3) {
	    result.outValue=Math.round(v/0.0001609344)/10000;
	  } else if (v<15) {
	    result.outValue=Math.round(v/0.01609344)/100;
	  } else {
	    result.outValue=Math.round(v/1.609344);
	  }
	  result.inUnit="km/h";
	  result.outUnit="mph";
	  return result;
	},

	deg2all:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  if (MCE.pref_util.getPref('pref_celsius')) {
	    result.inUnit=String.fromCharCode(176)+"F";
	    result.outUnit=String.fromCharCode(176)+"C";
	    result.outValue=(v-32)/1.8;
	  } else {
	    result.inUnit=String.fromCharCode(176)+"C";
	    result.outUnit=String.fromCharCode(176)+"F";
	    result.outValue=v*1.8+32;
	  }
	  return result;
	},

	mps2all:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.inUnit="m/s";
	  if (MCE.pref_util.getPref('pref_metric')) {
	    result.outUnit="km/h";
	    result.outValue=v * 18 / 5;
	  } else {
	    result.outUnit="mph";
	    result.outValue=v * 3600 / 1610.3;
	  }
	  return result;
	},

	kmps2all:function(v)
	{
	  var result=MCE.converters.mps2all(v);
	  result.outValue*=1000;
	  result.inUnit='km/s';
	  return result;
	},

	litre2gallon:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v/3.785411784;
	  result.inUnit="L";
	  result.outUnit="gal";
	  return result;
	},

	gallon2litre:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v*3.785411784;
	  result.inUnit="gal";
	  result.outUnit="L";
	  return result;
	},

	ml2floz:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v/29.5735295625;
	  result.inUnit="ml";
	  result.outUnit="fl oz";
	  return result;
	},

	floz2ml:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v*29.5735295625;
	  result.inUnit="fl oz";
	  result.outUnit="ml";
	  return result;
	},

	pa2psi:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v/6894.75729;
	  result.inUnit="Pa";
	  result.outUnit="psi";
	  return result;
	},

	kpa2psi:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v/6.89475729;
	  result.inUnit="KPa";
	  result.outUnit="psi";
	  return result;
	},

	psi2kpa:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v*6.89475729;
	  result.inUnit="psi";
	  result.outUnit="KPa";
	  return result;
	},

	cc2cuin:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v/16.387064;
	  result.inUnit="cm"+String.fromCharCode(179);
	  result.outUnit="in"+String.fromCharCode(179);
	  return result;
	},

	cuin2cc:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v*16.387064;
	  result.inUnit="in"+String.fromCharCode(179);
	  result.outUnit="cm"+String.fromCharCode(179);
	  return result;
	},

	mpg2lptm:function(v)
	{
	  var result=new Object();
	  result.only_show_converted=true;
	  result.not_numerical=true;
	  var oU=new Number(235.2/v);
	  var oI=new Number(282.5/v);
	  result.inValue=v;
	  result.inUnit="mpg";
	  // var out=out.toFixed(-val_e+2);
	  result.outCustom=v+"mp(US)g = "+oU.toFixed(2)+" L/100 km; "+v+" mp(Imp)g = "+oI.toFixed(2)+" L/100 km";
	  return result;
	},

	internal_time:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  // Removing the final "?" kills "7:00 pm"
	  var re = /([0-9]{1,2})(:?([0-9]{2}))?(:[0-9]{2})?[\s]*(am|pm|AM|PM|a\.m\.|p\.m\.|A\.M\.|P\.M\.)?[\s]*([+\-0-9A-Z]{3,5})?/;
	  var regexp_result = re.exec(v);
	  if (regexp_result[3]==undefined) {
	    regexp_result[3]="00";
	  }
	  if (regexp_result[4]==undefined) {
	    regexp_result[4]='';
	  }
	  if (regexp_result[5]) { // 12h -> 24h
	    if (regexp_result[1]=='12') {
	      regexp_result[1]=0;
	    }
	    if (regexp_result[5].toLowerCase().match(/pm|p\.m\./)) {
	      regexp_result[1]=String(Math.abs(regexp_result[1])+12);
	    }
	    var end_result=regexp_result[1]+':'+regexp_result[3]+regexp_result[4];
	    result.inUnit='12h';
	    result.outUnit='24h';
	  } else { // 24h -> 12h
	    if (regexp_result[1]=='00' || regexp_result[1]=='0') {
	      regexp_result[1]=12;
	    }
	    if (regexp_result[1]>12) {
	      end_result=String(Math.abs(regexp_result[1])-12)+':'+regexp_result[3]+regexp_result[4]+' PM';
	    } else {
	      end_result=regexp_result[1]+':'+regexp_result[3]+regexp_result[4]+' AM';
	    }
	    result.inUnit='24h';
	    result.outUnit='12h';
	  }
	  result.outValue=end_result;
	  result.not_numerical=true;
	  result.only_show_converted=true;
	  var local_tz=MCE.tzUtil.get_local_offset();
	  var orig_tz=MCE.tzUtil.get_tz_offset(regexp_result[6]);
	  if (isNaN(orig_tz) || orig_tz==local_tz) {
	    // No timezone spec, or same timezone -- returning converted, if applicable
	    if (
	      (result.outUnit=='24h' && MCE.pref_util.getPref('pref_24h')) ||
	      (result.outUnit=='12h' && !MCE.pref_util.getPref('pref_24h'))
	    ) {
	      return result;
	    } else {
	      return {inhibit:true};
	    }
	  }
	  var hh=regexp_result[1]; // regexp_result[1] is ALWAYS 24h, see above
	  var mm=regexp_result[3];
	  var ss=regexp_result[4];
	  //var tz=regexp_result[5];
	  var delta_tz=local_tz-orig_tz; // to be added to the orig time in order to get local time
	  var delta_tz_hh=Math.floor(delta_tz);
	  var delta_tz_mm=(delta_tz-delta_tz_hh)*60;
	  var local_hh=1*hh;
	  var local_mm=1*delta_tz_mm+1*mm;
	  if (local_mm>59) {
	    local_hh++;
	    local_mm-=60;
	  }
	  if (local_mm<0) {
	    local_hh--;
	    local_mm+=60;
	  }
	  if (local_mm<10) {
	    local_mm="0"+local_mm;
	  }
	  local_hh+=delta_tz_hh;
	  var local_dd=0;
	  if (local_hh>23) {
	    local_dd++;
	    local_hh-=24;
	  }
	  if (local_hh<0) {
	    local_dd--;
	    local_hh+=24;
	  }
	  if (local_dd==0) {
	    local_dd='';
	  } else if (local_dd==1) {
	    local_dd='/+1';
	  } else {
	    local_dd='/-1';
	  }
	  result.inUnit='remote TZ';
	  result.outUnit='local TZ';
	  if (MCE.pref_util.getPref('pref_24h')) {
	    result.outValue=local_hh+':'+local_mm+ss+local_dd;
	    return result;
	  } else {
	    if (local_hh==0) {
	      local_hh=12;
	    }
	    if (local_hh>12) {
	      local_hh-=12;
	      var local_ampm=' PM';
	    } else {
	      var local_ampm=' AM';
	    }
	    result.outValue=local_hh+':'+local_mm+ss+local_dd+local_ampm;
	    return result;
	  }
	},

	sqft2sqm:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v*0.09290304;
	  result.inUnit="sq ft";
	  result.outUnit="m"+String.fromCharCode(178);
	  return result;
	},

	sqm2sqft:function(v)
	{
	  var result=new Object();
	  result.inValue=v;
	  result.outValue=v/0.09290304;
	  result.inUnit="m"+String.fromCharCode(178);
	  result.outUnit="sq ft";
	  return result;
	},

	nm2ftlb:function(v)
	{
	  var result=new Object();
	  //result.only_show_converted=true;
	  result.not_numerical=true;
	  result.inValue=v;
	  var outFt=v*0.7375621;
	  var outIn=v*0.7375621*12;
	  outFt=Math.round(outFt*1000)/1000;
	  outIn=Math.round(outIn*1000)/1000;
	  result.inUnit="Nm";
	  result.outCustom=outFt+" ft-lb; "+outIn+" in-lb";
	  return result;
	}
}

MCE.adjusters={
	/*

	  ====================================================================
	                             ADJUSTERS
	  ====================================================================

	  These functions perform order of magnitude adjustments within the same
	  units system, trying to guess the most intuitive unit. For instance
	  it would be counter-intuitive to show a distance of the order of
	  hundreds of kilometers in millimeters.
	  
	*/

	metricWeight:function(o)
	{
	  if (o.outUnit!="kg") {
	    if (o.outUnit=='mg') {
	      return o;
	    }
	    return o;
	  }
	  var tv=o.outValue;
	  if ((tv>=1) && (tv<1000)) {
	    return o;
	  }
	  if (tv>=1000) {
	    o.outValue/=1000;
	    o.outUnit="t";
	    return o;
	  }
	  if (tv>=0.001) {
	    o.outValue*=1000;
	    o.outUnit="g";
	    return o;
	  }
	  o.outValue*=1000000;
	  o.outUnit="mg";
	  return o;
	},

	// I'm not familiar enough with the imperial units, these may not be the most
	// intuitive choices; I hope they're acceptable though -- if not, you're free
	// to send me a patch any time! :)
	imperialDistance:function(o)
	{
	  if (o.outUnit!="in") {
	    if (o.outUnit=="mil") {
	      return o;
	    }
	    return o;
	  }
	  var tv=o.outValue;
	  if (tv<(1.0/32.0)) {
	    o.outValue=tv*100;
	    o.outUnit="mil";
	    return o;
	  }
	  if (tv>=5280*36) {
	    o.outValue=tv/12/52.8/100;
	    o.outUnit="mi";
	    return o;
	  }
	  if (tv>=36000) {
	    o.outCustom=MCE.util.smartRound(tv/36)+" yd ("+MCE.util.smartRound(tv/12/52.8/100)+" mi)";
	    return o;
	  }
	  if (tv>=36) {
	    o.outCustom=MCE.util.smartRound(tv/36)+" yd ("+MCE.util.smartRound(tv/12)+" ft)";
	    return o;
	  }
	  if (MCE.pref_util.getPref('pref_in_fract')) {
	    var tv32=Math.round(tv*32) % 32;
	    if (tv32) {
	      if (tv32==16) {
		var fraction="1/2";
	      } else if (!tv32 % 8) {
		var fraction=(tv32/8)+"/4";
	      } else if (!tv32 % 4) {
		var fraction=(tv32/4)+"/8";
	      } else if (!tv32 % 2) {
		var fraction=(tv32/2)+"/16";
	      } else {
		var fraction=tv32+"/32";
	      }
	    }
	    tv=Math.floor(tv);
	    var inches=tv % 12;
	    tv=(tv-inches)/12;
	    o.outCustom=tv+"'"+inches;
	    if (fraction) {
	      o.outCustom=o.outCustom+"-"+fraction+"\"";
	    } else if (tv==0) {
	      o.outCustom=inches+"\"";
	    } else {
	      o.outCustom=o.outCustom+"\"";
	    }
	    return o;
	  } else {
	    o.outValue=tv;
	    o.outUnit='in';
	    return o;
	  }
	},

	metricDistance:function(o)
	{
	  if (o.outUnit!="m") {
	    if (o.outUnit==String.fromCharCode(181)+"m") {
	      return o;
	    }
	    return o;
	  }
	  var tv=o.outValue;
	  if ((tv>=1) && (tv<1000)) {
	    return o;
	  }
	  if (tv>=1000) {
	    o.outValue/=1000;
	    o.outUnit="km";
	    return o;
	  }
	  if (tv>=0.01) {
	    o.outValue*=100;
	    o.outUnit="cm";
	    return o;
	  }
	  if (tv>=0.0001) {
	    o.outValue*=1000;
	    o.outUnit="mm";
	    return o;
	  }
	  o.outValue*=1000000;
	  o.outUnit=String.fromCharCode(181)+"m";
	  return o;
	},

	imperialVolume:function(o)
	{
	  if ((o.outUnit=='fl oz') && (o.outValue>80)) {
	    o.outUnit='gal';
	    o.outValue/=128;
	  }
	  if ((o.outUnit=='gal') && (o.outValue<0.625)) {
	    o.outUnit='fl oz';
	    o.outValue*=128;
	  }
	  return o;
	},

	metricVolume:function(o)
	{
	  if ((o.outUnit=='ml') && (o.outValue>1000)) {
	    o.outUnit='L';
	    o.outValue/=1000;
	  }
	  if ((o.outUnit=='L') && (o.outValue<1)) {
	    o.outUnit='ml';
	    o.outValue*=1000;
	  }
	  return o;
	}
}

}
